@@ -2,14 +2,14 @@
  *
  *  Name:       common.h
  *
- *  Version:    2.1-1
+ *  Version:    2.4-2
  *
  *  Purpose:    Common definitions for RNetCDF functions
  *
  *  Author:     Pavel Michna (rnetcdf-devel@bluewin.ch)
  *              Milton Woods (miltonjwoods@gmail.com)
  *
- *  Copyright:  (C) 2004-2019 Pavel Michna, Milton Woods
+ *  Copyright:  (C) 2004-2020 Pavel Michna, Milton Woods
  *
  *=============================================================================*
  *
@@ -40,10 +40,6 @@
   #define NC_MAX_ATOMIC_TYPE NC_STRING
 #endif
 
-#define RRETURN(object) { R_nc_unprotect (); return (object); }
-
-#define RERROR(msg) { R_nc_error (msg); return NULL; }
-
 #define NA_SIZE SIZE_MAX
 
 /* Definition of missing value used by bit64 package */
@@ -54,18 +50,6 @@ static const char RNC_EDATALEN[]="Not enough data", \
   RNC_EDATATYPE[]="Incompatible data for external type", \
   RNC_ETYPEDROP[]="Unsupported external type";
 
-/* Protect an object from garbage collection by R */
-SEXP
-R_nc_protect (SEXP obj);
-
-/* Unprotect all objects to enable garbage collection by R */
-void
-R_nc_unprotect (void);
-
-/* Raise an error in R */
-void
-R_nc_error(const char *msg);
-
 /* If status is a netcdf error, raise an R error with a suitable message,
    otherwise return to caller. */
 int
